/*
 * PLL.h
 *
 *  Created on: May 2, 2018
 *      Author: Shane
 */

#ifndef PLL_H_
#define PLL_H_

#include <stdint.h>
#include <stddef.h>
#include <math.h>
#include <stdio.h>

/* Driver Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/UART.h>
#include <ti/drivers/ADC.h>

/* Example/Board Header files */
#include "Board.h"

#define MSGSIZE 4

SPI_Handle myInitSPI(void);
UART_Handle myInitUART(void);
void init_PLL(SPI_Handle spi, unsigned short * checksums);
float tune_PLL(float freq, SPI_Handle spi);
int PLL_TX(unsigned char b1, unsigned char b2, unsigned char b3, unsigned char b4, SPI_Handle spi);


SPI_Handle myInitSPI(void) {

    SPI_Params  spiParams;

    SPI_Params_init(&spiParams);               // Initialize SPI parameters
    spiParams.dataSize = 32;                   // 32 bit data size
    spiParams.frameFormat = SPI_POL0_PHA0;     // PLL SPI slave mode
    spiParams.bitRate = 1000000;              // SPI Clock rate

    return SPI_open(Board_SPI0, &spiParams);
}


UART_Handle myInitUART(void) {

    UART_Params uartParams;

     /* Create a UART with data processing off. */
    UART_Params_init(&uartParams);
    uartParams.writeDataMode = UART_DATA_BINARY;
    uartParams.readDataMode = UART_DATA_BINARY;
    uartParams.readReturnMode = UART_RETURN_FULL;
    uartParams.readEcho = UART_ECHO_OFF;
    uartParams.baudRate = 115200;

    return UART_open(Board_UART0, &uartParams);
}


// Executes SPI transmission to PLL
int PLL_TX(unsigned char b1, unsigned char b2, unsigned char b3, unsigned char b4, SPI_Handle spi){
    unsigned short int chksm;                                      // Variable used in check sum
    uint8_t byte1 = b1;
    uint8_t byte2 = b2;
    uint8_t byte3 = b3;
    uint8_t byte4 = b4;

    uint8_t transmitBuffer[MSGSIZE] = {0};
    uint8_t receiveBuffer[MSGSIZE] = {0};

    SPI_Transaction spiTransaction;
    spiTransaction.count = 1;  //
    spiTransaction.txBuf = transmitBuffer;
    spiTransaction.rxBuf = receiveBuffer;

    /* load transmit buffer */
    transmitBuffer[0] = byte4;
    transmitBuffer[1] = byte3;
    transmitBuffer[2] = byte2;
    transmitBuffer[3] = byte1;

    /*  send data over SPI to PLL   */
    bool transferOK = SPI_transfer(spi, &spiTransaction);
    if (!transferOK) {
        // Error in SPI or transfer already in progress.
    }

    chksm = byte1 + byte2 + byte3 + byte4;
    return chksm;
}

//tunes PLL to initial frequency of 250 MHz
//to write to PLL via SPI: low write enable bit, 6 bits register address, 24 bits register data = 31 bits
//last 16 bits of writes to register 5 forwarded to VCO subsystem MSB first:
//9 bits register data, 4 bits register address, 3 bits VCO ID (000 always)
//0 bit meaningless
//SEN high before SCLK means HMC SPI mode
void init_PLL(SPI_Handle spi, unsigned short * checksums){
    unsigned short checksum1, checksum2, checksum3, checksum4, checksum5, checksum6, checksum7 = 0;
    checksum1 = PLL_TX(0b00001110,0b00000000,0b00000010,0b10011010, spi);           //&H7 &H14D
    checksum2 = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi);           //&HC &H0  No Exact Frequency Comparison
    checksum3 = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi);           //&HC &H0  Ibid
    checksum4 = PLL_TX(0b00000000,0b00010100,0b11110010,0b11101010, spi);           //&H0 &HA7975 Read Only A7975 = Chip ID - ignore
    checksums[0] = checksum1 + checksum2 + checksum3 + checksum4;   // Sum is 714

    checksum1 = PLL_TX(0b00000010,0b00000000,0b00000000,0b00000100, spi);           //&H1 &H2    using SPI PLL enable bit
    checksum2 = PLL_TX(0b00000100,0b00000000,0b00000000,0b00000010, spi);           //&H2 &H1    Reference Divider R = 1
    checksum3 = PLL_TX(0b00000110,0b00000000,0b00000000,0b11110000, spi);           //&H3 &H78    Nint = 120
    checksum4 = PLL_TX(0b00001010,0b00000000,0b00101100,0b01010000, spi);           //&H5 &H1628 SubRegister 5
    checksums[1] = checksum1 + checksum2 + checksum3 + checksum4;   // Sum is 986

    checksum1 = PLL_TX(0b00001010,0b00000000,0b11000001,0b01000000, spi);           //&H5 &H60A0 SubRegister 4
    checksum2 = PLL_TX(0b00001010,0b00000000,0b00000000,0b00000000, spi);           //&H5 &H0    SubRegister 0
    checksum3 = PLL_TX(0b00001100,0b00000110,0b00011110,0b10010100, spi);           //&H6 &H30F4A
    checksum4 = PLL_TX(0b00001110,0b00000000,0b01000010,0b10011010, spi);           //&H7 &H214D Will try relocking if Lock Detect fails (only tries once)
    checksums[2] = checksum1 + checksum2 + checksum3 + checksum4; // Sum is 1693

    checksum1 = PLL_TX(0b00010001,0b10000011,0b01111101,0b11111110, spi);           //&H8 &HC1BEFF
    checksum2 = PLL_TX(0b00010010,0b10101000,0b11111111,0b11111110, spi);           //&H9 &H547FFF
    checksum3 = PLL_TX(0b00010100,0b00000000,0b01000000,0b10001110, spi);           //&HA &H02047 FSM = 50MHz (Ref clock), 256 measurement periods for Vtune accuracy of +/-98KHz
    checksum4 = PLL_TX(0b00010110,0b00001111,0b10000000,0b01000010, spi);           //&HB &H7C021
    checksums[3] = checksum1 + checksum2 + checksum3 + checksum4;   // Sum is 3372

    checksum1 = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi);           //&HC &H0   No Exact Frequency Comparison
    checksum2 = PLL_TX(0b00011010,0b00000000,0b00000000,0b00000000, spi);           //&HD &H0   Read Only - ignore
    checksum3 = PLL_TX(0b00011100,0b00000000,0b00000000,0b00000000, spi);           //&HE &H0   Read Only - ignore
    checksum4 = PLL_TX(0b00011110,0b00000000,0b00000001,0b10000010, spi);           //&HF &H1   Lock Detect Output on SDO
    checksums[4] = checksum1 + checksum2 + checksum3 + checksum4;   // Sum is ?

    checksum1 = PLL_TX(0b00100000,0b00000000,0b00000001,0b01100100, spi);           //&H10 &HB2  Read Only - ignore
    checksum2 = PLL_TX(0b00100010,0b00010000,0b00000000,0b00000100, spi);           //&H11 &H80002 Read Only - ignore
    checksum3 = PLL_TX(0b00100100,0b00000000,0b00000000,0b00000110, spi);           //&H12 &H3    Read Only - ignore (bit 1 = lock detect status)
    checksum4 = PLL_TX(0b00100110,0b00000000,0b00100100,0b10110010, spi);           //&H13 &H1259 Read Only - ignore
    checksums[5] = checksum1 + checksum2 + checksum3 + checksum4;   // Sum is 3963

    checksum1 = PLL_TX(0b00001000,0b00000000,0b00000000,0b00000000, spi);           //&H4 &H0  Nfrac = 0
    checksum2 = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi);           //&HC &H0  No Exact Frequency Comparison
    checksum3 = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi);           //&HC &H0  Ibid
    checksum4 = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi);           //&HC &H0  Ibid
    checksum5 = PLL_TX(0b00000100,0b00000000,0b00000000,0b00000010, spi);           //&H2 &H1    Reference Divider R = 1
    checksum6 = PLL_TX(0b00001010,0b00000000,0b01010001,0b00110000, spi);           //&H5 &H2898
    checksums[6] = checksum1 + checksum2 + checksum3 + checksum4 + checksum5 + checksum6;   // Sum is ?

    checksum1 = PLL_TX(0b00001010,0b00000000,0b00000000,0b00000000, spi);           //&H5 &H0
    checksum2 = PLL_TX(0b00000110,0b00000000,0b00000000,0b11110000, spi);           //&H3 &H78
    checksum3 = PLL_TX(0b00001010,0b00000000,0b00000000,0b00000000, spi);           //&H5 &H0
    checksum4 = PLL_TX(0b00001010,0b00000000,0b00000000,0b00000000, spi);           //&H4 &H0
    checksum5 = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi);           //&HC &H0  No Exact Frequency Comparison
    checksum6 = PLL_TX(0b00001010,0b00000000,0b11001100,0b00100000, spi);           //&H5 &H6610 SubRegister 2 f0/2 mode
    checksum7 = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi);           //&HC &H0  No Exact Frequency Comparison
    checksums[7] = checksum1 + checksum2 + checksum3 + checksum4 + checksum5 + checksum6 + checksum7;   // Sum is ?
    unsigned int i;
    for(i = 0; i < 7; i++){
        checksums[8] += checksums[i];
    }
}

// Set the frequency of PLL
float tune_PLL(float freq, SPI_Handle spi){  //freq in MHz
    unsigned short dumcheck;
    freq = freq - 110;           //nominal IF of 110 MHz
    float k1;
    long int k;
    long int f_osc = 25;         //MHz
    long int R_div = 1;
    long int f_pd = f_osc/R_div;
    uint32_t Nint;
    uint32_t Nfrac;
    float pll_freq;    //MHz
    //find values to write to PLL registers
    if(freq > 3000){
        k1 = 0.5;
        Nint = floor(k1*freq/f_pd);
        Nfrac = round((k1*freq/f_pd-Nint)*16777216); //16777216 = 2^24
    }else{
        k = floor(3000/freq);
        if(k > 1){
            k = k - (k % 2);
        }
        Nint = floor(k*freq/f_pd);
        Nfrac = round((k*freq/f_pd-Nint)*16777216); //16777216 = 2^24
    }
    //Nfrac = 24 bits for PLL
    //Nint = 19 bits for PLL
    //k = 6 bits for PLL
    Nfrac = Nfrac & 0x00FFFFFF;   //keep last 24 bits only
    Nint = Nint & 0x0007FFFF;     //keep last 19 bits only
    k = k & 0x3F;
    //parsing through Nfrac, Nint, k for SPI transmission
    //Nfrac = 24 bits 23:0, Nint = 19 bits 18:0
    uint8_t Nfrac0 = 0b00001000 | ((Nfrac >> 23) & 0x01);  //0 = W, address = &H4, MSB of Nfrac
    uint8_t Nfrac1 = (Nfrac >> 15) & 0xFF;  //bits 23:15
    uint8_t Nfrac2 = (Nfrac >> 7) & 0xFF;   //bits 14:7
    uint8_t Nfrac3 = Nfrac & 0xFE;          //bits 6:0 + 0
    //no Nint0 since only 19 bit data
    uint8_t Nint1 = (Nint >> 15) & 0x0F;    //0000 + bits 18:15
    uint8_t Nint2 = (Nint >> 7) & 0xFF;     //bits 14:7
    uint8_t Nint3 = Nint & 0xFE;            //bits 6:0 + 0
    //k subregister only has 6 bits of data
    uint8_t kdiv = 0b11000000 | k;          //11 + bits 5:0
    //write to registers to change fout
    //last 16 bits of writes to register 0x05 forwarded to VCO subsystem MSB first:
    //9 bits register data, 4 bits register address, 3 bits VCO ID (= 000 always)
    if(freq > 3000){
        pll_freq = f_pd*(Nint+Nfrac/2^24)*2;
        dumcheck = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi); //&HC &H0
        dumcheck = PLL_TX(0b00001010,0b00000001,0b11000001,0b00100000, spi); //&H5 &H6090 set kdiv = 1
        dumcheck = PLL_TX(0b00001010,0b00000001,0b01011010,0b00000000, spi); //&H5 &HAD00 using high freq sub band (also try &H0)
        dumcheck = PLL_TX(0b00010010,0b10101000,0b11111111,0b11111110, spi); //&H9 &H547FFF charge pump
        dumcheck = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi); //&HC &H0
        dumcheck = PLL_TX(0b00001100,0b00000110,0b00011110,0b10010100, spi); //&H6 &H30F4A
        dumcheck = PLL_TX(0b00001100,0b00000110,0b00011110,0b10010100, spi); //&H6 &H30F4A
        dumcheck = PLL_TX(0b00000100,0b00000000,0b00000000,0b00000010, spi); //&H2 &H1 set reference divider to 1
        dumcheck = PLL_TX(0b00001010,0b00000000,0b01000000,0b00110000, spi); //&H5 &H2018 set to doubler mode
        dumcheck = PLL_TX(0b00001010,0b00000000,0b00000000,0b00000000, spi); //&H5 &H0
        dumcheck = PLL_TX(0b00000110,Nint1,Nint2,Nint3, spi);                //&H3 Set Nint, integer reg to freq
        dumcheck = PLL_TX(0b00001010,0b00000000,0b00000000,0b00000000, spi); //&H5 &H0
        dumcheck = PLL_TX(Nfrac0,Nfrac1,Nfrac2,Nfrac3, spi);                 //&H4 Set Nfrac, freq fractional part
        dumcheck = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi); //&HC &H0
    }else{
        pll_freq = f_pd*(Nint+Nfrac/2^24)/k;
        dumcheck = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi); //&HC &H0
        if((k == 1) || (k == 2)){
            dumcheck = PLL_TX(0b00001010,0b00000001,kdiv,0b00100000, spi);   //&H5 &H6090 set divider accordingly set kdiv value
        }else{
            dumcheck = PLL_TX(0b00001010,0b00000000,kdiv,0b00100000, spi);   //&H5 &H6090 set divider accordingly set kdiv value
        }
        dumcheck = PLL_TX(0b00001010,0b00000000,0b01000010,0b00000000, spi); //&H5 &H2100 tuning default (also try &H0)
        dumcheck = PLL_TX(0b00010010,0b10101000,0b11111111,0b11111110, spi); //&H9 &H547FFF charge pump
        dumcheck = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi); //&HC &H0
        dumcheck = PLL_TX(0b00001100,0b00000110,0b00011110,0b10010100, spi); //&H6 &H30F4A Sigma Delta Bypass for Integer mode
        dumcheck = PLL_TX(0b00001100,0b00000110,0b00011110,0b10010100, spi); //&H6 &H30F4A
        dumcheck = PLL_TX(0b00000100,0b00000000,0b00000000,0b00000010, spi); //&H2 &H1 set reference divider to 1
        dumcheck = PLL_TX(0b00001010,0b00000000,0b01010001,0b00110000, spi); //&H5 &H2898 set to fundamental mode
        dumcheck = PLL_TX(0b00001010,0b00000000,0b00000000,0b00000000, spi); //&H5 &H0
        dumcheck = PLL_TX(0b00000110,Nint1,Nint2,Nint3, spi);                //&H3 Set Nint, integer reg to freq
        dumcheck = PLL_TX(0b00001010,0b00000000,0b00000000,0b00000000, spi); //&H5 &H0
        dumcheck = PLL_TX(Nfrac0,Nfrac1,Nfrac2,Nfrac3, spi);                 //&H4 Set Nfrac, freq fractional part
        dumcheck = PLL_TX(0b00011000,0b00000000,0b00000000,0b00000000, spi); //&HC &H0
    }
    (void)dumcheck;
    return(pll_freq);
}









#endif /* PLL_H_ */
