/*
 * adc_single_channel.h
 *
 *  Created on: May 2, 2018
 *      Author: Shane
 */

#ifndef ADC_SINGLE_CHANNEL_H_
#define ADC_SINGLE_CHANNEL_H_


#include <stdint.h>
#include <stddef.h>

/* POSIX Header files */
#include <pthread.h>

/* Driver Header files */
#include <ti/drivers/ADC.h>
#include <ti/display/Display.h>

/* Example/Board Header files */
#include "Board.h"

/* ADC sample count */
#define ADC_SAMPLE_COUNT  (3125)    //averaging samples in 50 ms period
#define TotalAverages    (1)

#define THREADSTACKSIZE   (768)

/* ADC conversion result variables */
uint16_t adcValue0;
uint32_t adcValue0MicroVolt;
uint16_t adcValue1[ADC_SAMPLE_COUNT];
uint32_t adcValue1MicroVolt[ADC_SAMPLE_COUNT];

static Display_Handle display;

/*
 *  ======== threadFxn0 ========
 *  Open an ADC instance and get a sampling result from a one-shot conversion.
 */
void *threadFxn0(void *arg0)
{
    ADC_Handle   adc;
    ADC_Params   params;
    int_fast16_t res;

    ADC_Params_init(&params);
    adc = ADC_open(Board_ADC0, &params);

    if (adc == NULL) {
        //Display_printf(display, 0, 0, "Error initializing ADC channel 0\n");
        while (1);
    }

    /* Blocking mode conversion */
    res = ADC_convert(adc, &adcValue0);

    if (res == ADC_STATUS_SUCCESS) {

        adcValue0MicroVolt = ADC_convertRawToMicroVolts(adc, adcValue0);

        //Display_printf(display, 0, 0, "ADC channel 0 convert result: %d uV\n", adcValue0MicroVolt);
    }
    else {
        //Display_printf(display, 0, 0, "ADC channel 0 convert failed\n");
    }

    ADC_close(adc);

    return (NULL);
}

/*
 *  ======== threadFxn1 ========
 *  Open a ADC handle and get a array of sampling results after
 *  calling several conversions.
 */
void *threadFxn1(void *arg0)
{
    uint16_t     i;
    ADC_Handle   adc;
    ADC_Params   params;
    int_fast16_t res;
    uint32_t avgVal;
    uint32_t chunk = ADC_SAMPLE_COUNT/TotalAverages;
    uint32_t totalVolts = 0;

    ADC_Params_init(&params);
    adc = ADC_open(Board_ADC1, &params);

    if (adc == NULL) {
        Display_printf(display, 0, 0, "Error initializing ADC channel 1\n");
        while (1);
    }
    /* Modified Code for averaging */
    for (i = 1; i < ADC_SAMPLE_COUNT+1; i++) {
        res = ADC_convert(adc, &adcValue1[i]);

        if (res == ADC_STATUS_SUCCESS) {

            adcValue1MicroVolt[i] = ADC_convertRawToMicroVolts(adc, adcValue1[i]);
            totalVolts = totalVolts + adcValue1MicroVolt[i];

            if (i % chunk == 0){
                avgVal = totalVolts/chunk;
                Display_printf(display, 0, 0, "ADC channel 1 convert result (%d): %d uV\n", i,
                    avgVal);
                totalVolts = 0;
            }
        }
        else {
            Display_printf(display, 0, 0, "ADC channel 1 convert failed (%d)\n", i);
        }
        /* Original Example Code
         * for (i = 0; i < ADC_SAMPLE_COUNT; i++) {
            res = ADC_convert(adc, &adcValue1[i]);

            if (res == ADC_STATUS_SUCCESS) {

                adcValue1MicroVolt[i] = ADC_convertRawToMicroVolts(adc, adcValue1[i]);

                Display_printf(display, 0, 0, "ADC channel 1 convert result (%d): %d uV\n", i,
                                   adcValue1MicroVolt[i]);
            }
            else {
                Display_printf(display, 0, 0, "ADC channel 1 convert failed (%d)\n", i);
         }*/
    }

    ADC_close(adc);

    return (NULL);
}


#endif /* ADC_SINGLE_CHANNEL_H_ */
